@@ -26,6 +26,7 @@ from c3.utils.qt_utils import (
     cliffords_decomp,
     cliffords_decomp_xId,
     single_length_RB,
+    cliffords_string,
 )
 
 fidelities = dict()
@@ -39,70 +40,6 @@ def fid_reg_deco(func):
     return func
 
 
-@fid_reg_deco
-def iswap_transfer(U_dict: dict, index, dims, eval, proj=True):
-    infids = []
-    indeces = [0, 1, dims[2], dims[2] + 1]
-    for indx in indeces:
-        psi_init = [0] * np.prod(dims)
-        psi_init[indx] = 1
-        psi_0 = tf.Variable(psi_init, dtype=tf.complex128, shape=[np.prod(dims), 1])
-        infid = state_transfer_infid(U_dict, "Id:iSWAP", index, dims, psi_0, proj)
-        infids.append(infid)
-    # for indx in [0,2,4,6]:
-    #     psi_init = [0] * 8
-    #     psi_init[indx] = 1
-    #     psi_0 = tf.Variable(psi_init, dtype=tf.complex128, shape=[8,1])
-    #     infid = state_transfer_infid(U_dict, "iSWAP:Id", index, dims, psi_0, proj)
-    #     infids.append(infid)
-    return tf.reduce_mean(infids)
-
-
-@fid_reg_deco
-def iswap_comp_sub(U_dict: dict, index, dims, eval, proj=True):
-    dim = dims[1]
-    # U_ideal = tf.Variable(
-    #     perfect_gate("iSWAP", [0,1], [dim,dim]),
-    #     dtype=tf.complex128, projection =
-    # )
-
-    # proj = [[1,0],
-    #         [0,1]]
-    # lvls = 2
-    # while dim > lvls:
-    #     proj = proj.append([0,0])
-    #     lvls = lvls + 1
-    # p = np.kron(proj, proj)
-    # print(p)
-    # p = tf.Variable(p, dtype=tf.complex128)
-    # U_comp = p.T@U@p
-    # print(U)
-    # print(U_comp)
-    U = {}
-    U["iSWAP"] = U_dict["Id:iSWAP"][: dim ** 2, : dim ** 2]
-    infid = unitary_infid(U, "iSWAP", [0, 1], [dim, dim], proj=proj)
-    return infid
-
-
-@fid_reg_deco
-def iswap_leakage(U_dict: dict, index, dims, eval, proj=True):
-    indeces = np.append(
-        np.append(np.arange(2, dims[1]), np.arange(2, dims[1]) + dims[2]),
-        np.arange(dims[1] * dims[2], np.prod(dims)),
-    )
-    psi_init = [0] * np.prod(dims)
-    psi_init[1] = 1
-    psi_0 = tf.Variable(psi_init, dtype=tf.complex128, shape=[np.prod(dims), 1])
-    psi_actual = tf.matmul(U_dict["Id:iSWAP"], psi_0)
-    leakage = 0
-    for indx in indeces:  # fix indeces
-        psi = [0] * np.prod(dims)
-        psi[indx] = 1
-        psi = tf.Variable(psi, dtype=tf.complex128, shape=[np.prod(dims), 1])
-        leakage = leakage + tf_ketket_fid(psi, psi_actual)
-    return leakage
-
-
 @fid_reg_deco
 def state_transfer_infid_set(U_dict: dict, index, dims, psi_0, proj=True):
     """
@@ -172,27 +109,6 @@ def state_transfer_infid(U_dict: dict, gate: str, index, dims, psi_0, proj: bool
     return infid
 
 
-# def population(U_dict: dict, lvl: int, gate: str):
-#     U = U_dict[gate]
-#     lvls = U.shape[0]
-#     psi_0 = tf.Variable(basis(lvls, 0), dtype=tf.complex128)
-#     psi_f = tf.Variable(basis(lvls, lvl), dtype=tf.complex128)
-#     psi_actual = tf.matmul(U, psi_0)
-#     overlap = tf_ketket_fid(psi_f, psi_actual)
-#     return overlap
-#
-#
-# def lindbladian_population(U_dict: dict, lvl: int, gate: str):
-#     U = U_dict[gate]
-#     lvls = int(np.sqrt(U.shape[0]))
-#     psi_0 = tf.Variable(basis(lvls, 0), dtype=tf.complex128)
-#     dv_0 = tf_dm_to_vect(tf_state_to_dm(psi_0))
-#     psi_f = tf.Variable(basis(lvls, lvl), dtype=tf.complex128)
-#     dv_actual = tf.matmul(U, dv_0)
-#     overlap = tf_dmket_fid(dv_actual, psi_f)
-#     return overlap
-
-
 @fid_reg_deco
 def unitary_infid(U_dict: dict, gate: str, index, dims, proj: bool):
     """
@@ -217,7 +133,6 @@ def unitary_infid(U_dict: dict, gate: str, index, dims, proj: bool):
         Unitary fidelity.
     """
     U = U_dict[gate]
-    # print(U)
     projection = "fulluni"
     fid_lvls = np.prod([dims[i] for i in index])
     if proj:
@@ -226,9 +141,7 @@ def unitary_infid(U_dict: dict, gate: str, index, dims, proj: bool):
     U_ideal = tf.Variable(
         perfect_gate(gate, index, dims, projection), dtype=tf.complex128
     )
-    # print(U_ideal)
     infid = 1 - tf_unitary_overlap(U, U_ideal, lvls=fid_lvls)
-    # print(gate, '  :  ', infid)
     return infid
 
 
@@ -462,47 +375,41 @@ def lindbladian_average_infid_set(U_dict: dict, index, dims, eval, proj=True):
 
 
 @fid_reg_deco
-def epc_analytical(U_dict: dict, index, dims, proj: bool):
-    # TODO make this work with new index and dims
-    gate = list(U_dict.keys())[0]
-    U = U_dict[gate]
-    num_gates = len(gate.split(":"))
-    lvls = int(U.shape[0] ** (1 / num_gates))
-    fid_lvls = lvls
-    if num_gates == 1:
+def epc_analytical(U_dict: dict, index, dims, proj: bool, cliffords=False):
+    # TODO check this work with new index and dims (double-check)
+    num_gates = len(dims)
+    if cliffords:
+        real_cliffords = evaluate_sequences(U_dict, [[C] for C in cliffords_string])
+    elif num_gates == 1:
         real_cliffords = evaluate_sequences(U_dict, cliffords_decomp)
     elif num_gates == 2:
         real_cliffords = evaluate_sequences(U_dict, cliffords_decomp_xId)
-    projection = "fulluni"
-    if proj:
-        projection = "wzeros"
-        fid_lvls = 2 ** num_gates
-    ideal_cliffords = perfect_cliffords(str(lvls), proj=projection, num_gates=num_gates)
+    ideal_cliffords = perfect_cliffords(lvls=[2] * num_gates, num_gates=num_gates)
     fids = []
     for C_indx in range(24):
         C_real = real_cliffords[C_indx]
-        C_ideal = ideal_cliffords[C_indx]
-        ave_fid = tf_average_fidelity(C_real, C_ideal, fid_lvls)
+        C_ideal = tf.constant(ideal_cliffords[C_indx], dtype=tf.complex128)
+        ave_fid = tf_average_fidelity(C_real, C_ideal, lvls=dims)
         fids.append(ave_fid)
     infid = 1 - tf_ave(fids)
     return infid
 
 
 @fid_reg_deco
-def lindbladian_epc_analytical(U_dict: dict, proj: bool):
-    real_cliffords = evaluate_sequences(U_dict, cliffords_decomp)
-    lvls = int(np.sqrt(real_cliffords[0].shape[0]))
-    projection = "fulluni"
-    fid_lvls = lvls
-    if proj:
-        projection = "wzeros"
-        fid_lvls = 2
-    ideal_cliffords = perfect_cliffords(str(lvls), projection)
+def lindbladian_epc_analytical(U_dict: dict, index, dims, proj: bool, cliffords=False):
+    num_gates = len(dims)
+    if cliffords:
+        real_cliffords = evaluate_sequences(U_dict, [[C] for C in cliffords_string])
+    elif num_gates == 1:
+        real_cliffords = evaluate_sequences(U_dict, cliffords_decomp)
+    elif num_gates == 2:
+        real_cliffords = evaluate_sequences(U_dict, cliffords_decomp_xId)
+    ideal_cliffords = perfect_cliffords(lvls=[2] * num_gates, num_gates=num_gates)
     fids = []
     for C_indx in range(24):
         C_real = real_cliffords[C_indx]
-        C_ideal = tf_super(tf.Variable(ideal_cliffords[C_indx], dtype=tf.complex128))
-        ave_fid = tf_superoper_average_fidelity(C_real, C_ideal, lvls=fid_lvls)
+        C_ideal = tf_super(tf.constant(ideal_cliffords[C_indx], dtype=tf.complex128))
+        ave_fid = tf_superoper_average_fidelity(C_real, C_ideal, lvls=dims)
         fids.append(ave_fid)
     infid = 1 - tf_ave(fids)
     return infid
@@ -550,7 +457,6 @@ def RB(
     lindbladian=False,
     padding="",
 ):
-    # print('Performing RB fit experiment.')
     gate = list(U_dict.keys())[0]
     U = U_dict[gate]
     dim = int(U.shape[0])
@@ -570,7 +476,7 @@ def RB(
         pop0s = []
         for U in Us:
             pops = populations(tf.matmul(U, psi_init), lindbladian)
-            pop0s.append(float(pops[0] + pops[1]))
+            pop0s.append(float(pops[0]))
         surv_prob.append(pop0s)
 
     def RB_fit(len, r, A, B):
@@ -611,40 +517,11 @@ def RB(
                 pop0s = []
                 for U in Us:
                     pops = populations(tf.matmul(U, psi_init), lindbladian)
-                    pop0s.append(float(pops[0] + pops[1]))
+                    pop0s.append(float(pops[0]))
                 surv_prob.append(pop0s)
             lengths = np.append(lengths, new_lengths)
     epc = 0.5 * (1 - r)
-    print("epc:", epc)
-    epg = 1 - ((1 - epc) ** (1 / 4))
-    print("epg:", epg)
-    # print('example seq: ', seqs[0])
-
-    # fig, ax = plt.subplots()
-    # ax.plot(lengths,
-    #         surv_prob,
-    #         marker='o',
-    #         color='red',
-    #         linestyle='None')
-    # ax.errorbar(lengths,
-    #             means,
-    #             yerr=stds,
-    #             color='blue',
-    #             marker='x',
-    #             linestyle='None')
-    # plt.title('RB results')
-    # plt.ylabel('Population in 0')
-    # plt.xlabel('\# Cliffords')
-    # plt.ylim(0, 1)
-    # plt.xlim(0, lengths[-1])
-    # fitted = RB_fit(lengths, r, A, B)
-    # ax.plot(lengths, fitted)
-    # plt.text(0.1, 0.1,
-    #          'r={:.4f}, A={:.3f}, B={:.3f}'.format(r, A, B),
-    #          size=16,
-    #          transform=ax.transAxes)
-    # plt.show()
-    # return epc, r, A, B, fig, ax
+    epg = 1 - ((1 - epc) ** (1 / 4))  # TODO: adjust to be mean length of
     return epg
 
 
@@ -674,7 +551,6 @@ def leakage_RB(
     logspace=False,
     lindbladian=False,
 ):
-    # print('Performing leakage RB fit experiment.')
     gate = list(U_dict.keys())[0]
     U = U_dict[gate]
     dim = int(U.shape[0])
